@@ -1,5 +1,5 @@
 
-set rcsId {$Id: demo,v 1.33 1998/04/20 18:41:37 jfontain Exp $}
+set rcsId {$Id: demo,v 1.36 1998/05/03 20:13:54 jfontain Exp $}
 
 if {[catch {package require stooop 3.5.1}]} {
 set rcsId {$Id: stooop.tcl,v 3.60 1998/04/18 20:43:14 jfontain Exp $}
@@ -652,33 +652,35 @@ class switched {
 }
 }
 
-if {[catch {package require tkpiechart 4.2}]} {
-set rcsId {$Id: pielabel.tcl,v 1.37 1998/03/28 20:42:15 jfontain Exp $}
+if {[catch {package require tkpiechart 5.0}]} {
+set rcsId {$Id: pielabel.tcl,v 1.38 1998/05/03 10:50:27 jfontain Exp $}
 
 class pieLabeler {
 
     set pieLabeler::(default,font) {Helvetica -12}
 
     proc pieLabeler {this canvas args} {
-        set pieLabeler::($this,canvas) $canvas
+        ::set pieLabeler::($this,canvas) $canvas
     }
 
     proc ~pieLabeler {this} {}
 
     proc link {this pie} {
-        set pieLabeler::($this,pie) $pie
+        ::set pieLabeler::($this,pie) $pie
     }
 
     virtual proc new {this slice args}
 
     virtual proc delete {this label}
 
-    virtual proc update {this label value}
+    virtual proc set {this label value}
 
     virtual proc selectState {this label {state {}}}
 
+    virtual proc update {this} {}
+
 }
-set rcsId {$Id: perilabel.tcl,v 1.36 1998/04/09 21:55:19 jfontain Exp $}
+set rcsId {$Id: perilabel.tcl,v 1.39 1998/05/03 18:15:53 jfontain Exp $}
 
 class piePeripheralLabeler {
 
@@ -713,21 +715,18 @@ class piePeripheralLabeler {
     }
 
     proc new {this slice args} {
-        set canvas $pieLabeler::($this,canvas)
+        ::set canvas $pieLabeler::($this,canvas)
 
-        set text [$canvas create text 0 0 -font $switched::($this,-smallfont) -tags pieLabeler($this)]
-        set box [$canvas bbox $text]
-        set smallTextHeight [expr {[lindex $box 3]-[lindex $box 1]}]
+        ::set text [$canvas create text 0 0 -font $switched::($this,-smallfont) -tags pieLabeler($this)]
+        ::set box [$canvas bbox $text]
+        ::set smallTextHeight [expr {[lindex $box 3]-[lindex $box 1]}]
 
         if {![info exists piePeripheralLabeler::($this,array)]} {
-            set box [$canvas bbox pie($pieLabeler::($this,pie))]
-            set piePeripheralLabeler::($this,array) [eval ::new canvasLabelsArray\
-                $canvas [lindex $box 0] [expr {[lindex $box 3]+(2*$switched::($this,-offset))+$smallTextHeight}]\
-                [expr {[lindex $box 2]-[lindex $box 0]}] -justify $switched::($this,-justify) -xoffset $switched::($this,-xoffset)\
-            ]
+            ::set piePeripheralLabeler::($this,array)\
+                [::new canvasLabelsArray $canvas 0 0 -justify $switched::($this,-justify) -xoffset $switched::($this,-xoffset)]
+            update $this
         }
-
-        set label [eval ::new canvasLabel $pieLabeler::($this,canvas) 0 0 $args\
+        ::set label [eval ::new canvasLabel $pieLabeler::($this,canvas) 0 0 $args\
             [list\
                 -style split -justify $switched::($this,-justify) -bulletwidth $switched::($this,-bulletwidth)\
                 -font $switched::($this,-font)\
@@ -737,9 +736,9 @@ class piePeripheralLabeler {
 
         $canvas addtag pieLabeler($this) withtag canvasLabelsArray($piePeripheralLabeler::($this,array))
 
-        set piePeripheralLabeler::($this,textItem,$label) $text
-        set piePeripheralLabeler::($this,slice,$label) $slice
-        set piePeripheralLabeler::($this,selected,$label) 0
+        ::set piePeripheralLabeler::($this,textItem,$label) $text
+        ::set piePeripheralLabeler::($this,slice,$label) $slice
+        ::set piePeripheralLabeler::($this,selected,$label) 0
 
         return $label
     }
@@ -748,15 +747,15 @@ class piePeripheralLabeler {
         return [expr {(2*($degrees/90))+(($degrees%90)!=0)}]
     }
 
-    set index 0
+    ::set index 0
     foreach anchor {w sw s se e ne n nw} {
-        set piePeripheralLabeler::(anchor,[anglePosition [expr {$index*45}]]) $anchor
+        ::set piePeripheralLabeler::(anchor,[anglePosition [expr {$index*45}]]) $anchor
         incr index
     }
     unset index anchor
 
-    proc update {this label value} {
-        set text $piePeripheralLabeler::($this,textItem,$label)
+    proc set {this label value} {
+        ::set text $piePeripheralLabeler::($this,textItem,$label)
         position $this $text $piePeripheralLabeler::($this,slice,$label)
         $pieLabeler::($this,canvas) itemconfigure $text -text $value
     }
@@ -765,17 +764,17 @@ class piePeripheralLabeler {
         variable PI
 
         slice::data $slice data
-        set midAngle [expr {$data(start)+($data(extent)/2.0)}]
-        set radians [expr {$midAngle*$PI/180}]
-        set x [expr {($data(xRadius)+$switched::($this,-offset))*cos($radians)}]
-        set y [expr {($data(yRadius)+$switched::($this,-offset))*sin($radians)}]
-        set angle [expr {round($midAngle)%360}]
+        ::set midAngle [expr {$data(start)+($data(extent)/2.0)}]
+        ::set radians [expr {$midAngle*$PI/180}]
+        ::set x [expr {($data(xRadius)+$switched::($this,-offset))*cos($radians)}]
+        ::set y [expr {($data(yRadius)+$switched::($this,-offset))*sin($radians)}]
+        ::set angle [expr {round($midAngle)%360}]
         if {$angle>180} {
-            set y [expr {$y-$data(height)}]
+            ::set y [expr {$y-$data(height)}]
         }
 
-        set canvas $pieLabeler::($this,canvas)
-        set coordinates [$canvas coords $text]
+        ::set canvas $pieLabeler::($this,canvas)
+        ::set coordinates [$canvas coords $text]
         $canvas move $text [expr {$data(xCenter)+$x-[lindex $coordinates 0]}] [expr {$data(yCenter)-$y-[lindex $coordinates 1]}]
         $canvas itemconfigure $text -anchor $piePeripheralLabeler::(anchor,[anglePosition $angle])
     }
@@ -799,11 +798,25 @@ class piePeripheralLabeler {
         } else {
             switched::configure $label -borderwidth 1
         }
-        set piePeripheralLabeler::($this,selected,$label) $selected
+        ::set piePeripheralLabeler::($this,selected,$label) $selected
+    }
+
+    proc update {this} {
+        ::set canvas $pieLabeler::($this,canvas)
+        ::set box [$canvas bbox pieGraphics($pieLabeler::($this,pie))]
+::set smallTextHeight 10
+        ::set array $piePeripheralLabeler::($this,array)
+        foreach {x y} [$canvas coords canvasLabelsArray($array)] {}
+        $canvas move canvasLabelsArray($array) [expr {[lindex $box 0]-$x}]\
+            [expr {[lindex $box 3]+(2*$switched::($this,-offset))+$smallTextHeight-$y}]
+        switched::configure $array -width [expr {[lindex $box 2]-[lindex $box 0]}]
+        foreach label [canvasLabelsArray::labels $array] {
+            position $this $piePeripheralLabeler::($this,textItem,$label) $piePeripheralLabeler::($this,slice,$label)
+        }
     }
 
 }
-set rcsId {$Id: boxlabel.tcl,v 1.34 1998/03/28 20:42:15 jfontain Exp $}
+set rcsId {$Id: boxlabel.tcl,v 1.36 1998/05/03 15:01:20 jfontain Exp $}
 
 class pieBoxLabeler {
 
@@ -834,19 +847,16 @@ class pieBoxLabeler {
 
     proc new {this slice args} {
         if {![info exists pieBoxLabeler::($this,array)]} {
-            set box [$pieLabeler::($this,canvas) bbox pie($pieLabeler::($this,pie))]
-            set pieBoxLabeler::($this,array) [::new canvasLabelsArray\
-                $pieLabeler::($this,canvas) [lindex $box 0] [expr {[lindex $box 3]+$switched::($this,-offset)}]\
-                [expr {[lindex $box 2]-[lindex $box 0]}] -xoffset $switched::($this,-xoffset)\
-            ]
+            ::set pieBoxLabeler::($this,array) [::new canvasLabelsArray $pieLabeler::($this,canvas) 0 0]
+            update $this
         }
-        set label [eval ::new canvasLabel $pieLabeler::($this,canvas) 0 0\
+        ::set label [eval ::new canvasLabel $pieLabeler::($this,canvas) 0 0\
             $args [list -justify $switched::($this,-justify) -font $switched::($this,-font)]\
         ]
         canvasLabelsArray::manage $pieBoxLabeler::($this,array) $label
         $pieLabeler::($this,canvas) addtag pieLabeler($this) withtag canvasLabelsArray($pieBoxLabeler::($this,array))
         switched::configure $label -text [switched::cget $label -text]:
-        set pieBoxLabeler::($this,selected,$label) 0
+        ::set pieBoxLabeler::($this,selected,$label) 0
         return $label
     }
 
@@ -855,7 +865,7 @@ class pieBoxLabeler {
         unset pieBoxLabeler::($this,selected,$label)
     }
 
-    proc update {this label value} {
+    proc set {this label value} {
         regsub {:.*$} [switched::cget $label -text] ": $value" text
         switched::configure $label -text $text
     }
@@ -869,17 +879,26 @@ class pieBoxLabeler {
         } else {
             switched::configure $label -borderwidth 1
         }
-        set pieBoxLabeler::($this,selected,$label) $selected
+        ::set pieBoxLabeler::($this,selected,$label) $selected
+    }
+
+    proc update {this} {
+        ::set canvas $pieLabeler::($this,canvas)
+        ::set box [$canvas bbox pieGraphics($pieLabeler::($this,pie))]
+        ::set array $pieBoxLabeler::($this,array)
+        foreach {x y} [$canvas coords canvasLabelsArray($array)] {}
+        $canvas move canvasLabelsArray($array) [expr {[lindex $box 0]-$x}] [expr {[lindex $box 3]+$switched::($this,-offset)-$y}]
+        switched::configure $array -width [expr {[lindex $box 2]-[lindex $box 0]}]
     }
 
 }
-set rcsId {$Id: canlabel.tcl,v 1.20 1998/03/28 08:38:12 jfontain Exp $}
+set rcsId {$Id: canlabel.tcl,v 1.21 1998/05/03 10:52:49 jfontain Exp $}
 
 class canvasLabel {
 
     proc canvasLabel {this canvas x y args} switched {$args} {
         set canvasLabel::($this,canvas) $canvas
-        set canvasLabel::($this,origin) [$canvas create line $x $y $x $y -fill {} -tags canvasLabel($this)]
+        set canvasLabel::($this,origin) [$canvas create image $x $y -tags canvasLabel($this)]
         set canvasLabel::($this,rectangle) [$canvas create rectangle 0 0 0 0 -tags canvasLabel($this)]
         set canvasLabel::($this,text) [$canvas create text 0 0 -tags canvasLabel($this)]
 
@@ -901,6 +920,7 @@ class canvasLabel {
             [list -foreground black black]\
             [list -justify left left]\
             [list -padding 2 2]\
+            [list -scale {1 1} {1 1}]\
             [list -stipple {} {}]\
             [list -style box box]\
             [list -text {} {}]\
@@ -921,6 +941,9 @@ class canvasLabel {
     proc set-foreground {this value} {
         $canvasLabel::($this,canvas) itemconfigure $canvasLabel::($this,text) -fill $value
     }
+    proc set-scale {this value} {
+        update $this
+    }
     proc set-stipple {this value} {
         $canvasLabel::($this,canvas) itemconfigure $canvasLabel::($this,rectangle) -stipple $value
     }
@@ -945,9 +968,7 @@ class canvasLabel {
         set rectangle $canvasLabel::($this,rectangle)
         set text $canvasLabel::($this,text)
 
-        set coordinates [$canvas coords $canvasLabel::($this,origin)]
-        set x [lindex $coordinates 0]
-        set y [lindex $coordinates 1]
+        foreach {x y} [$canvas coords $canvasLabel::($this,origin)] {}
 
         set border [$canvas itemcget $rectangle -width]
         set textBox [$canvas bbox $text]
@@ -973,17 +994,17 @@ class canvasLabel {
         set yDelta [expr {([string match n* $anchor]-[string match s* $anchor])*$halfHeight}]
         $canvas move $rectangle $xDelta $yDelta
         $canvas move $text $xDelta $yDelta
+        eval $canvas scale canvasLabel($this) $x $y $switched::($this,-scale)
     }
 
 }
-set rcsId {$Id: labarray.tcl,v 1.16 1998/03/28 20:37:16 jfontain Exp $}
+set rcsId {$Id: labarray.tcl,v 1.18 1998/05/03 15:00:55 jfontain Exp $}
 
 class canvasLabelsArray {
 
-    proc canvasLabelsArray {this canvas x y width args} switched {$args} {
+    proc canvasLabelsArray {this canvas x y args} switched {$args} {
         set canvasLabelsArray::($this,canvas) $canvas
-        set canvasLabelsArray::($this,width) [winfo fpixels $canvas $width]
-        set canvasLabelsArray::($this,origin) [$canvas create line $x $y $x $y -fill {} -tags canvasLabelsArray($this)]
+        set canvasLabelsArray::($this,origin) [$canvas create image $x $y -tags canvasLabelsArray($this)]
         set canvasLabelsArray::($this,labels) {}
         switched::complete $this
     }
@@ -997,6 +1018,7 @@ class canvasLabelsArray {
         return [list\
             [list -justify left left]\
             [list -xoffset 0 0]\
+            [list -width 100]\
         ]
     }
 
@@ -1008,6 +1030,19 @@ class canvasLabelsArray {
         "
     }
 
+    proc set-width {this value} {
+        set canvasLabelsArray::($this,width) [winfo fpixels $canvasLabelsArray::($this,canvas) $value]
+        update $this
+    }
+
+    proc update {this} {
+        set index 0
+        foreach label $canvasLabelsArray::($this,labels) {
+            position $this $label $index
+            incr index
+        }
+    }
+
     proc manage {this label} {
         $canvasLabelsArray::($this,canvas) addtag canvasLabelsArray($this) withtag canvasLabel($label)
         set index [llength $canvasLabelsArray::($this,labels)]
@@ -1031,9 +1066,8 @@ class canvasLabelsArray {
     proc position {this label index} {
         set canvas $canvasLabelsArray::($this,canvas)
 
-        set coordinates [$canvas coords $canvasLabelsArray::($this,origin)]
-        set x [expr {[lindex $coordinates 0]+(($index%2?1:-1)*$switched::($this,-xoffset))}]
-        set y [lindex $coordinates 1]
+        foreach {x y} [$canvas coords $canvasLabelsArray::($this,origin)] {}
+        set x [expr {$x+(($index%2?1:-1)*$switched::($this,-xoffset))}]
         set coordinates [$canvas bbox canvasLabel($label)]
         set y [expr {$y+(($index/2)*([lindex $coordinates 3]-[lindex $coordinates 1]))}]
 
@@ -1052,8 +1086,8 @@ class canvasLabelsArray {
             }
         }
         switched::configure $label -anchor $anchor
-        set coordinates [$canvas coords canvasLabel($label)]
-        $canvas move canvasLabel($label) [expr {$x-[lindex $coordinates 0]}] [expr {$y-[lindex $coordinates 1]}]
+        foreach {xDelta yDelta} [$canvas coords canvasLabel($label)] {}
+        $canvas move canvasLabel($label) [expr {$x-$xDelta}] [expr {$y-$yDelta}]
     }
 
     proc labels {this} {
@@ -1061,7 +1095,7 @@ class canvasLabelsArray {
     }
 
 }
-set rcsId {$Id: slice.tcl,v 1.32 1998/04/09 21:56:03 jfontain Exp $}
+set rcsId {$Id: slice.tcl,v 1.35 1998/05/03 18:08:55 jfontain Exp $}
 
 
 class slice {
@@ -1069,12 +1103,10 @@ class slice {
 }
 
 proc slice::slice {this canvas x y radiusX radiusY start extent args} switched {$args} {
-
     set slice::($this,canvas) $canvas
     set slice::($this,start) 0
     set slice::($this,radiusX) $radiusX
     set slice::($this,radiusY) $radiusY
-
     switched::complete $this
     complete $this $x $y
     update $this $start $extent
@@ -1092,6 +1124,7 @@ proc slice::options {this} {
         [list -bottomcolor {} {}]\
         [list -deletecommand {} {}]\
         [list -height 0 0]\
+        [list -scale {1 1} {1 1}]\
         [list -topcolor {} {}]\
     ]
 }
@@ -1106,6 +1139,12 @@ foreach option {-bottomcolor -height -topcolor} {
 
 proc slice::set-deletecommand {this value} {}
 
+proc slice::set-scale {this value} {
+    if {$switched::($this,complete)} {
+        update $this $slice::($this,start) $slice::($this,extent)
+    }
+}
+
 proc slice::normalizedAngle {value} {
     while {$value>=180} {
         set value [expr {$value-360}]
@@ -1121,9 +1160,7 @@ proc slice::complete {this x y} {
     set radiusX $slice::($this,radiusX)
     set radiusY $slice::($this,radiusY)
     set bottomColor $switched::($this,-bottomcolor)
-
-    set slice::($this,origin) [$canvas create line -$radiusX -$radiusY -$radiusX -$radiusY -fill {} -tags slice($this)]
-
+    set slice::($this,origin) [$canvas create image -$radiusX -$radiusY -tags slice($this)]
     if {$switched::($this,-height)>0} {
         set slice::($this,startBottomArcFill) [$canvas create arc\
             0 0 0 0 -style chord -extent 0 -fill $bottomColor -outline $bottomColor -tags slice($this)\
@@ -1142,36 +1179,30 @@ proc slice::complete {this x y} {
         set slice::($this,endLeftLine) [$canvas create line 0 0 0 0 -tags slice($this)]
         set slice::($this,endRightLine) [$canvas create line 0 0 0 0 -tags slice($this)]
     }
-
     set slice::($this,topArc) [$canvas create arc\
         -$radiusX -$radiusY $radiusX $radiusY -fill $switched::($this,-topcolor) -tags slice($this)\
     ]
-
     $canvas move slice($this) [expr {$x+$radiusX}] [expr {$y+$radiusY}]
 }
 
 proc slice::update {this start extent} {
     set canvas $slice::($this,canvas)
-
-    set coordinates [$canvas coords slice($this)]
-
+    set coordinates [$canvas coords $slice::($this,origin)]
     set radiusX $slice::($this,radiusX)
     set radiusY $slice::($this,radiusY)
-    $canvas coords $slice::($this,origin) -$radiusX -$radiusY $radiusX $radiusY
+    $canvas coords $slice::($this,origin) -$radiusX -$radiusY
     $canvas coords $slice::($this,topArc) -$radiusX -$radiusY $radiusX $radiusY
-
     set extent [maximum 0 $extent]
     if {$extent>=360} {
-        set extent 359.9999999999999
+        set extent [expr {360-pow(10,-$::tcl_precision+3)}]
     }
     $canvas itemconfigure $slice::($this,topArc)\
         -start [set slice::($this,start) [normalizedAngle $start]] -extent [set slice::($this,extent) $extent]
-
     if {$switched::($this,-height)>0} {
         updateBottom $this
     }
-
     $canvas move slice($this) [expr {[lindex $coordinates 0]+$radiusX}] [expr {[lindex $coordinates 1]+$radiusY}]
+    eval $canvas scale slice($this) $coordinates $switched::($this,-scale)
 }
 
 proc slice::updateBottom {this} {
@@ -1281,28 +1312,29 @@ proc slice::data {this arrayName} {
 
     set data(start) $slice::($this,start)
     set data(extent) $slice::($this,extent)
-    set data(xRadius) $slice::($this,radiusX)
-    set data(yRadius) $slice::($this,radiusY)
-    set coordinates [$slice::($this,canvas) coords $slice::($this,origin)]
-    set data(xCenter) [expr {[lindex $coordinates 0]+$data(xRadius)}]
-    set data(yCenter) [expr {[lindex $coordinates 1]+$data(yRadius)}]
-    set data(height) $switched::($this,-height)
+    foreach {x y} $switched::($this,-scale) {}
+    set data(xRadius) [expr {$x*$slice::($this,radiusX)}]
+    set data(yRadius) [expr {$y*$slice::($this,radiusY)}]
+    set data(height) [expr {$y*$switched::($this,-height)}]
+    foreach {x y} [$slice::($this,canvas) coords $slice::($this,origin)] {}
+    set data(xCenter) [expr {$x+$data(xRadius)}]
+    set data(yCenter) [expr {$y+$data(yRadius)}]
 }
 
 class slice {
     proc maximum {a b} {return [expr {$a>$b?$a:$b}]}
 }
 
-set rcsId {$Id: objselec.tcl,v 1.3 1998/04/20 14:51:14 jfontain Exp $}
+set rcsId {$Id: selector.tcl,v 1.1 1998/04/24 10:46:30 jfontain Exp $}
 
 
-class objectSelection {
+class selector {
 
-    proc objectSelection {this args} switched {$args} {
+    proc selector {this args} switched {$args} {
         switched::complete $this
     }
 
-    proc ~objectSelection {this} {
+    proc ~selector {this} {
         variable ${this}selected
 
         catch {::unset ${this}selected}
@@ -1316,19 +1348,19 @@ class objectSelection {
 
     proc set-selectcommand {this value} {}
 
-    proc set {this ids selected} {
+    proc set {this indices selected} {
         variable ${this}selected
 
         ::set select {}
         ::set deselect {}
-        foreach id $ids {
-            if {[info exists ${this}selected($id)]&&($selected==[::set ${this}selected($id)])} continue
+        foreach index $indices {
+            if {[info exists ${this}selected($index)]&&($selected==[::set ${this}selected($index)])} continue
             if {$selected} {
-                lappend select $id
-                ::set ${this}selected($id) 1
+                lappend select $index
+                ::set ${this}selected($index) 1
             } else {
-                lappend deselect $id
-                ::set ${this}selected($id) 0
+                lappend deselect $index
+                ::set ${this}selected($index) 0
             }
         }
         update $this $select $deselect
@@ -1344,68 +1376,52 @@ class objectSelection {
         }
     }
 
-    proc unset {this ids} {
+    proc unset {this indices} {
         variable ${this}selected
 
-        foreach id $ids {
-            ::unset ${this}selected($id)
+        foreach index $indices {
+            ::unset ${this}selected($index)
         }
     }
 
 
-    proc add {this ids} {
-        set $this $ids 0
+    proc add {this indices} {
+        set $this $indices 0
     }
 
-    proc remove {this ids} {
-        unset $this $ids
+    proc remove {this indices} {
+        unset $this $indices
     }
 
-    proc select {this ids} {
+    proc select {this indices} {
         clear $this
-        set $this $ids 1
-        ::set objectSelection::($this,lastSelected) [lindex $ids end]
+        set $this $indices 1
+        ::set selector::($this,lastSelected) [lindex $indices end]
     }
 
-    proc deselect {this ids} {
-        set $this $ids 0
+    proc deselect {this indices} {
+        set $this $indices 0
     }
 
-    proc toggle {this ids} {
+    proc toggle {this indices} {
         variable ${this}selected
 
         ::set select {}
         ::set deselect {}
-        foreach id $ids {
-            if {[::set ${this}selected($id)]} {
-                lappend deselect $id
-                ::set ${this}selected($id) 0
+        foreach index $indices {
+            if {[::set ${this}selected($index)]} {
+                lappend deselect $index
+                ::set ${this}selected($index) 0
             } else {
-                lappend select $id
-                ::set ${this}selected($id) 1
-                ::set objectSelection::($this,lastSelected) $id
+                lappend select $index
+                ::set ${this}selected($index) 1
+                ::set selector::($this,lastSelected) $index
             }
         }
         update $this $select $deselect
     }
 
-    proc extend {this id} {
-        variable ${this}selected
-
-        if {[info exists objectSelection::($this,lastSelected)]} {
-            ::set list [lsort -integer [array names ${this}selected]]
-            ::set last [lsearch -exact $list $objectSelection::($this,lastSelected)]
-            ::set index [lsearch -exact $list $id]
-            clear $this
-            if {$index>$last} {
-                set $this [lrange $list $last $index] 1
-            } else {
-                set $this [lrange $list $index $last] 1
-            }
-        } else {
-            select $this $id
-        }
-    }
+    virtual proc extend {this index} {}
 
     proc clear {this} {
         variable ${this}selected
@@ -1413,27 +1429,62 @@ class objectSelection {
         set $this [array names ${this}selected] 0
     }
 
-    proc selected {this} {
+    virtual proc selected {this} {
         variable ${this}selected
 
         ::set list {}
-        foreach id [array names ${this}selected] {
-            if {[::set ${this}selected($id)]} {
-                lappend list $id
+        foreach index [array names ${this}selected] {
+            if {[::set ${this}selected($index)]} {
+                lappend list $index
             }
         }
-        return [lsort -integer $list]
+        return $list
+    }
+
+    virtual proc list {this} {
+        variable ${this}selected
+
+        return [array names ${this}selected]
+    }
+}
+
+set rcsId {$Id: objselec.tcl,v 1.4 1998/04/24 10:51:45 jfontain Exp $}
+
+
+class objectSelector {
+
+    proc objectSelector {this args} selector {$args} {}
+
+    proc ~objectSelector {this} {}
+
+
+    proc extend {this id} {
+        if {[info exists selector::($this,lastSelected)]} {
+            set list [list $this]
+            set last [lsearch -exact $list $selector::($this,lastSelected)]
+            set index [lsearch -exact $list $id]
+            selector::clear $this
+            if {$index>$last} {
+                selector::set $this [lrange $list $last $index] 1
+            } else {
+                selector::set $this [lrange $list $index $last] 1
+            }
+        } else {
+            selector::select $this $id
+        }
+    }
+
+    proc selected {this} {
+        return [lsort -integer [selector::_selected $this]]
     }
 
     proc list {this} {
-        variable ${this}selected
-
-        return [lsort -integer [array names ${this}selected]]
+        return [lsort -integer [selector::_list $this]]
     }
 }
-set rcsId {$Id: pie.tcl,v 1.64 1998/04/20 20:17:25 jfontain Exp $}
+set rcsId {$Id: pie.tcl,v 1.70 1998/05/03 18:39:41 jfontain Exp $}
 
-package provide tkpiechart 4.2
+package provide tkpiechart 5.0
 
 class pie {
     set pie::(colors) {#7FFFFF #7FFF7F #FF7F7F #FFFF7F #7F7FFF #FFBF00 #BFBFBF #FF7FFF #FFFFFF}
@@ -1450,6 +1501,9 @@ proc pie::pie {this canvas x y width height args} switched {$args} {
     set pie::($this,colorIndex) 0
     set pie::($this,slices) {}
 
+    set pie::($this,xScale) 1
+    set pie::($this,yScale) 1
+
     switched::complete $this
     complete $this
 }
@@ -1471,6 +1525,7 @@ proc pie::options {this} {
         [list -title {} {}]\
         [list -titlefont {} {}]\
         [list -titleoffset 2 2]\
+        [list -scale {1 1} {1 1}]\
     ]
 }
 
@@ -1482,6 +1537,18 @@ foreach option {-background -colors -labeler -selectable -title -titlefont -titl
     "
 }
 
+proc pie::set-scale {this value} {
+    foreach {xScale yScale} $value {}
+    if {($xScale==$pie::($this,xScale))&&($yScale==$pie::($this,yScale))} return
+    switched::configure $pie::($this,backgroundSlice) -scale $value
+    foreach slice $pie::($this,slices) {
+        switched::configure $slice -scale $value
+    }
+    pieLabeler::update $pie::($this,labeler)
+    set pie::($this,xScale) $xScale
+    set pie::($this,yScale) $yScale
+}
+
 proc pie::set-thickness {this value} {
     if {$switched::($this,complete)} {
         error {option -thickness cannot be set dynamically}
@@ -1524,10 +1591,10 @@ proc pie::newSlice {this {text {}}} {
     set color [lindex $switched::($this,-colors) $pie::($this,colorIndex)]
     set pie::($this,colorIndex) [expr {($pie::($this,colorIndex)+1)%[llength $switched::($this,-colors)]}]
 
-    set coordinates [$canvas coords slice($pie::($this,backgroundSlice))]
+    foreach {x y} [$canvas coords slice($pie::($this,backgroundSlice))] {}
 
     set slice [new slice\
-        $canvas [lindex $coordinates 0] [lindex $coordinates 1] $pie::($this,radiusX) $pie::($this,radiusY) $start 0\
+        $canvas $x $y $pie::($this,radiusX) $pie::($this,radiusY) $start 0 -scale $switched::($this,-scale)\
         -height $pie::($this,thickness) -topcolor $color -bottomcolor [tkDarken $color 60]\
     ]
     $canvas addtag pie($this) withtag slice($slice)
@@ -1544,16 +1611,16 @@ proc pie::newSlice {this {text {}}} {
 
     if {$switched::($this,-selectable)} {
         if {![info exists pie::($this,selector)]} {
-            set pie::($this,selector) [new objectSelection -selectcommand "pie::setLabelsState $this"]
+            set pie::($this,selector) [new objectSelector -selectcommand "pie::setLabelsState $this"]
         }
         set selector $pie::($this,selector)
-        objectSelection::add $selector $label
-        $canvas bind canvasLabel($label) <ButtonRelease-1> "objectSelection::select $selector $label"
-        $canvas bind slice($slice) <ButtonRelease-1> "objectSelection::select $selector $label"
-        $canvas bind canvasLabel($label) <Control-ButtonRelease-1> "objectSelection::toggle $selector $label"
-        $canvas bind slice($slice) <Control-ButtonRelease-1> "objectSelection::toggle $selector $label"
-        $canvas bind canvasLabel($label) <Shift-ButtonRelease-1> "objectSelection::extend $selector $label"
-        $canvas bind slice($slice) <Shift-ButtonRelease-1> "objectSelection::extend $selector $label"
+        selector::add $selector $label
+        $canvas bind canvasLabel($label) <ButtonRelease-1> "selector::select $selector $label"
+        $canvas bind slice($slice) <ButtonRelease-1> "selector::select $selector $label"
+        $canvas bind canvasLabel($label) <Control-ButtonRelease-1> "selector::toggle $selector $label"
+        $canvas bind slice($slice) <Control-ButtonRelease-1> "selector::toggle $selector $label"
+        $canvas bind canvasLabel($label) <Shift-ButtonRelease-1> "selector::extend $selector $label"
+        $canvas bind slice($slice) <Shift-ButtonRelease-1> "selector::extend $selector $label"
     }
 
     return $slice
@@ -1572,7 +1639,7 @@ proc pie::deleteSlice {this slice} {
     }
     pieLabeler::delete $pie::($this,labeler) $pie::($this,sliceLabel,$slice)
     if {$switched::($this,-selectable)} {
-        objectSelection::remove $pie::($this,selector) $pie::($this,sliceLabel,$slice)
+        selector::remove $pie::($this,selector) $pie::($this,sliceLabel,$slice)
     }
     unset pie::($this,sliceLabel,$slice)
 }
@@ -1587,9 +1654,9 @@ proc pie::sizeSlice {this slice unitShare {valueToDisplay {}}} {
     slice::update $slice [expr {$slice::($slice,start)-$growth}] $newExtent
 
     if {[string length $valueToDisplay]>0} {
-        pieLabeler::update $pie::($this,labeler) $pie::($this,sliceLabel,$slice) $valueToDisplay
+        pieLabeler::set $pie::($this,labeler) $pie::($this,sliceLabel,$slice) $valueToDisplay
     } else {
-        pieLabeler::update $pie::($this,labeler) $pie::($this,sliceLabel,$slice) $unitShare
+        pieLabeler::set $pie::($this,labeler) $pie::($this,sliceLabel,$slice) $unitShare
     }
 
     set value [expr {-1*$growth}]
@@ -1667,11 +1734,6 @@ set slice22 [pie::newSlice $pie2]
 $canvas move pie($pie1) 10 40
 $canvas move pie($pie2) 240 40
 
-set extent [$canvas bbox all]
-$canvas configure\
-    -width [expr [lindex $extent 2]-[lindex $extent 0]]\
-    -height [expr [lindex $extent 3]-[lindex $extent 1]]
-
 for {set index 1} {$index<=2} {incr index} {
     $canvas bind pie([set pie$index]) <ButtonPress-1> "
         set xLast($index) %x
@@ -1719,5 +1781,20 @@ proc refresh {} {
     after 3000 refresh
 }
 
+set initialWidth 450
+set initialHeight 250
+
+proc resize {width height} {
+    set scale [list\
+        [expr {double($width)/$::initialWidth}]\
+        [expr {double($height)/$::initialHeight}]\
+    ]
+    switched::configure $::pie1 -scale $scale
+    switched::configure $::pie2 -scale $scale
+    $::canvas configure -scrollregion [$::canvas bbox all]
+}
+
+$canvas configure -width $initialWidth -height $initialHeight
+bind $canvas <Configure> "resize %w %h"
 refresh
 
