@@ -1,6 +1,6 @@
-set rcsId {$Id: pie.tcl,v 1.64 1998/04/20 20:17:25 jfontain Exp $}
+set rcsId {$Id: pie.tcl,v 1.70 1998/05/03 18:39:41 jfontain Exp $}
 
-package provide tkpiechart 4.2
+package provide tkpiechart 5.0
 
 class pie {
     set pie::(colors) {#7FFFFF #7FFF7F #FF7F7F #FFFF7F #7F7FFF #FFBF00 #BFBFBF #FF7FFF #FFFFFF}
@@ -17,6 +17,9 @@ proc pie::pie {this canvas x y width height args} switched {$args} {
     set pie::($this,colorIndex) 0
     set pie::($this,slices) {}
 
+    set pie::($this,xScale) 1
+    set pie::($this,yScale) 1
+
     switched::complete $this
     complete $this                                                  ;# wait till all options have been set for initial configuration
 }
@@ -39,6 +42,7 @@ proc pie::options {this} {
         [list -title {} {}]\
         [list -titlefont {} {}]\
         [list -titleoffset 2 2]\
+        [list -scale {1 1} {1 1}]\
     ]
 }
 
@@ -51,6 +55,18 @@ foreach option {-background -colors -labeler -selectable -title -titlefont -titl
     "
 }
 
+proc pie::set-scale {this value} {
+    foreach {xScale yScale} $value {}
+    if {($xScale==$pie::($this,xScale))&&($yScale==$pie::($this,yScale))} return                                        ;# no change
+    switched::configure $pie::($this,backgroundSlice) -scale $value                             ;# update scale of background slice,
+    foreach slice $pie::($this,slices) {
+        switched::configure $slice -scale $value                                                                 ;# and other slices
+    }
+    pieLabeler::update $pie::($this,labeler)                          ;# finally update labels now that pie graphics are in position
+    set pie::($this,xScale) $xScale
+    set pie::($this,yScale) $yScale
+}
+
 proc pie::set-thickness {this value} {
     if {$switched::($this,complete)} {
         error {option -thickness cannot be set dynamically}
@@ -94,11 +110,11 @@ proc pie::newSlice {this {text {}}} {
     set pie::($this,colorIndex) [expr {($pie::($this,colorIndex)+1)%[llength $switched::($this,-colors)]}]  ;# circle through colors
 
     # make sure slice is positioned correctly in case pie was moved
-    set coordinates [$canvas coords slice($pie::($this,backgroundSlice))]
+    foreach {x y} [$canvas coords slice($pie::($this,backgroundSlice))] {}
 
     # darken slice top color by 40% to obtain bottom color, as it is done for Tk buttons shadow, for example
     set slice [new slice\
-        $canvas [lindex $coordinates 0] [lindex $coordinates 1] $pie::($this,radiusX) $pie::($this,radiusY) $start 0\
+        $canvas $x $y $pie::($this,radiusX) $pie::($this,radiusY) $start 0 -scale $switched::($this,-scale)\
         -height $pie::($this,thickness) -topcolor $color -bottomcolor [tkDarken $color 60]\
     ]
     $canvas addtag pie($this) withtag slice($slice)
@@ -115,16 +131,16 @@ proc pie::newSlice {this {text {}}} {
 
     if {$switched::($this,-selectable)} {                                             ;# toggle select state at every button release
         if {![info exists pie::($this,selector)]} {                                                  ;# create selector if necessary
-            set pie::($this,selector) [new objectSelection -selectcommand "pie::setLabelsState $this"]
+            set pie::($this,selector) [new objectSelector -selectcommand "pie::setLabelsState $this"]
         }
         set selector $pie::($this,selector)
-        objectSelection::add $selector $label
-        $canvas bind canvasLabel($label) <ButtonRelease-1> "objectSelection::select $selector $label"
-        $canvas bind slice($slice) <ButtonRelease-1> "objectSelection::select $selector $label"
-        $canvas bind canvasLabel($label) <Control-ButtonRelease-1> "objectSelection::toggle $selector $label"
-        $canvas bind slice($slice) <Control-ButtonRelease-1> "objectSelection::toggle $selector $label"
-        $canvas bind canvasLabel($label) <Shift-ButtonRelease-1> "objectSelection::extend $selector $label"
-        $canvas bind slice($slice) <Shift-ButtonRelease-1> "objectSelection::extend $selector $label"
+        selector::add $selector $label
+        $canvas bind canvasLabel($label) <ButtonRelease-1> "selector::select $selector $label"
+        $canvas bind slice($slice) <ButtonRelease-1> "selector::select $selector $label"
+        $canvas bind canvasLabel($label) <Control-ButtonRelease-1> "selector::toggle $selector $label"
+        $canvas bind slice($slice) <Control-ButtonRelease-1> "selector::toggle $selector $label"
+        $canvas bind canvasLabel($label) <Shift-ButtonRelease-1> "selector::extend $selector $label"
+        $canvas bind slice($slice) <Shift-ButtonRelease-1> "selector::extend $selector $label"
     }
 
     return $slice
@@ -144,7 +160,7 @@ proc pie::deleteSlice {this slice} {
     # finally delete label last so that other labels may eventually be repositionned according to remaining slices placement
     pieLabeler::delete $pie::($this,labeler) $pie::($this,sliceLabel,$slice)
     if {$switched::($this,-selectable)} {
-        objectSelection::remove $pie::($this,selector) $pie::($this,sliceLabel,$slice)
+        selector::remove $pie::($this,selector) $pie::($this,sliceLabel,$slice)
     }
     unset pie::($this,sliceLabel,$slice)
 }
@@ -160,9 +176,9 @@ proc pie::sizeSlice {this slice unitShare {valueToDisplay {}}} {
     slice::update $slice [expr {$slice::($slice,start)-$growth}] $newExtent                                        ;# grow clockwise
 
     if {[string length $valueToDisplay]>0} {                  ;# update label after slice for it may need slice latest configuration
-        pieLabeler::update $pie::($this,labeler) $pie::($this,sliceLabel,$slice) $valueToDisplay
+        pieLabeler::set $pie::($this,labeler) $pie::($this,sliceLabel,$slice) $valueToDisplay
     } else {
-        pieLabeler::update $pie::($this,labeler) $pie::($this,sliceLabel,$slice) $unitShare
+        pieLabeler::set $pie::($this,labeler) $pie::($this,sliceLabel,$slice) $unitShare
     }
 
     set value [expr {-1*$growth}]                                                               ;# finally move the following slices
