@@ -1,6 +1,6 @@
-set rcsId {$Id: stooop.tcl,v 3.57 1998/03/30 08:35:38 jfontain Exp $}
+set rcsId {$Id: stooop.tcl,v 3.60 1998/04/18 20:43:14 jfontain Exp $}
 
-package provide stooop 3.4
+package provide stooop 3.5.1
 
 catch {rename proc _proc}            ;# rename proc before it is overloaded, ignore error in case of multiple inclusion of this file
 
@@ -57,15 +57,15 @@ namespace eval ::stooop {
         # use local variable for identifier because new can be invoked recursively
         if {[scan $classOrId %u dummy]==0} {                                                            ;# first argument is a class
             set constructor ${classOrId}::[namespace tail $classOrId]                                   ;# generate constructor name
-            # generate fully qualified class namespace name
-            set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
             # we could detect here whether class was ever declared but that would prevent stooop packages to load properly, because
             # constructor would not be invoked and thus class source file never sourced
-            set fullClass([set id [incr newId]]) $fullName
             # invoke the constructor for the class with optional arguments in caller's variable context so that object creation is
             # transparent and that array names as constructor parameters work with a simple upvar
-            uplevel $constructor $id $args
-        } else {  ;# first argument is an object identifier (unsigned integer) , copy source object to new object of identical class
+            # note: if class is in a package, the class namespace code is loaded here, as the first object of the class is created
+            uplevel $constructor [set id [incr newId]] $args
+            # generate fully qualified class namespace name now that we are sure that class namespace code has been invoked
+            set fullClass($id) [namespace qualifiers [uplevel namespace which -command $constructor]]
+        } else {   ;# first argument is an object identifier (unsigned integer), copy source object to new object of identical class
             if {[catch {set fullClass([set id [incr newId]]) $fullClass($classOrId)}]} {
                 error "invalid object identifier $classOrId"
             }
@@ -450,9 +450,10 @@ if {[llength [array names ::env STOOOP*]]>0} {             ;# if one or more env
 
             if {[scan $classOrId %u dummy]==0} {                                                        ;# first argument is a class
                 set constructor ${classOrId}::[namespace tail $classOrId]                               ;# generate constructor name
+                catch {$constructor}              ;# force loading in case class is in a package so namespace commands work properly
                 set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
             } else {                                                                       ;# first argument is an object identifier
-                set fullName $fullClass($classOrId)
+                set fullName $fullClass($classOrId)                         ;# class code, if from a package, must already be loaded
             }
             if {[info exists interface($fullName)]} {
                 error "class $fullName with pure virtual procedures should not be instanciated"
@@ -507,6 +508,7 @@ if {[llength [array names ::env STOOOP*]]>0} {             ;# if one or more env
 
         debugInformation class qualifiedClass procedure qualifiedProcedure this
         if {![info exists this]} return                                                    ;# static procedure, no checking possible
+        if {[string compare $procedure constructor]==0} return   ;# in constructor, checking useless since object is not yet created
         if {![info exists fullClass($this)]} {
             error "$this is not a valid object identifier"
         }
